// Copyright (C) 2020-2021  Nexedi SA and Contributors.
//                          Kirill Smelkov <kirr@nexedi.com>
//
// This program is free software: you can Use, Study, Modify and Redistribute
// it under the terms of the GNU General Public License version 3, or (at your
// option) any later version, as published by the Free Software Foundation.
//
// You can also Link and Combine this program with other software covered by
// the terms of any of the Free Software licenses or any of the Open Source
// Initiative approved licenses and Convey the resulting work. Corresponding
// source of such a combination shall include the source code for all other
// software used.
//
// This program is distributed WITHOUT ANY WARRANTY; without even the implied
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// See COPYING file for full licensing terms.
// See https://www.nexedi.com/licensing for rationale and options.

// Package xurl complements standard package net/url.
package xurl

import (
	"fmt"
	"net/url"
)

// ParseQuery parses query string into regular map.
// see also url.ParseQuery
func ParseQuery(query string) (map[string]string, error) {
	qv, err := url.ParseQuery(query)
	if err != nil {
		return nil, err
	}
	q := map[string]string{}
	for k, vv := range qv {
		if len(vv) == 0 {
			return nil, fmt.Errorf("parameter %q without value", k)
		}
		if len(vv) != 1 {
			return nil, fmt.Errorf("duplicate parameter %q ", k)
		}
		q[k] = vv[0]
	}
	return q, nil
}
