// Copyright (C) 2023  Nexedi SA and Contributors.
//
// This program is free software: you can Use, Study, Modify and Redistribute
// it under the terms of the GNU General Public License version 3, or (at your
// option) any later version, as published by the Free Software Foundation.
//
// You can also Link and Combine this program with other software covered by
// the terms of any of the Free Software licenses or any of the Open Source
// Initiative approved licenses and Convey the resulting work. Corresponding
// source of such a combination shall include the source code for all other
// software used.
//
// This program is distributed WITHOUT ANY WARRANTY; without even the implied
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// See COPYING file for full licensing terms.
// See https://www.nexedi.com/licensing for rationale and options.

package xfilepath

import (
	"os/user"
	"testing"

	"lab.nexedi.com/kirr/neo/go/internal/xtesting"
)

func TestExpandUser(t *testing.T) {
	X := xtesting.FatalIf(t)
	xhomeOf := func(username string) string {
		t.Helper()
		home, err := homeOf(username); X(err)
		return home
	}
	home := xhomeOf("")
	homeRoot := xhomeOf("root")
	me, err := user.Current(); X(err)
	testv := []struct{ path, expandOk string }{
		{"", ""},
		{"~", home},
		{"~/", home},
		{"~" + me.Username, home},
		{"~/abc", home + "/abc"},
		{"~root/", homeRoot},
		{"~root", homeRoot},
		{"abc/", "abc/"},
		{"abc/~", "abc/~"},
		{"/abc/", "/abc/"},
	}
	for _, tCase := range testv {
		expanded := ExpandUser(tCase.path)
		if expanded != tCase.expandOk {
			t.Errorf("%s  -> %s  ; want %s", tCase.path, expanded, tCase.expandOk)
		}
	}
}
