// Copyright (C) 2016-2021  Nexedi SA and Contributors.
//                          Kirill Smelkov <kirr@nexedi.com>
//
// This program is free software: you can Use, Study, Modify and Redistribute
// it under the terms of the GNU General Public License version 3, or (at your
// option) any later version, as published by the Free Software Foundation.
//
// You can also Link and Combine this program with other software covered by
// the terms of any of the Free Software licenses or any of the Open Source
// Initiative approved licenses and Convey the resulting work. Corresponding
// source of such a combination shall include the source code for all other
// software used.
//
// This program is distributed WITHOUT ANY WARRANTY; without even the implied
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// See COPYING file for full licensing terms.
// See https://www.nexedi.com/licensing for rationale and options.

package neonet
// syntax sugar for atomic load/store to raise signal/noise in logic

import (
	"io"
	"sync/atomic"

	"github.com/philhofer/fwd"
)


type atomic32 struct {
	v int32	// struct member so `var a atomic32; if a == 0 ...` does not work
}

func (a *atomic32) Get() int32 {
	return atomic.LoadInt32(&a.v)
}

func (a *atomic32) Set(v int32) {
	atomic.StoreInt32(&a.v, v)
}

func (a *atomic32) Add(δ int32) int32 {
	return atomic.AddInt32(&a.v, δ)
}


// xbufReader provides fwd.Reader with io.LimitedReader inserted underneath it.
type xbufReader struct {
	*fwd.Reader
	Limit       io.LimitedReader // .Reader reads through .limit
}

func newXBufReader(r io.Reader, n int64) *xbufReader {
	rxbuf := &xbufReader{Limit: io.LimitedReader{R: r, N: n}}
	rxbuf.Reader = fwd.NewReader(&rxbuf.Limit)
	return rxbuf
}
